/*
 * RTParentPeerImpl.java
 *
 * Created on May 26, 2005, 5:39 AM
 *
 */

package peer;

import java.util.concurrent.*;
import scheduler.*;
import worker.*;
import server.*;
import client.*;
import peer.RTPeer;

/**
 *
 * @author Sean Halle
 */
public class RTParentPeerImpl extends RTPeer
 {
   protected BlockingQueue toClientQ;
   protected BlockingQueue toSchedQ;
   protected BlockingQueue toWorkerQ;
   protected RTClient      theClient;
   protected RTServer      theServer;
   protected RTWorker      theWorker;
   protected RTScheduler   theScheduler;   
     
   /**
    * Creates a new instance of RTParentPeerImpl 
    * 
    * RTParentPeerImpl is for the user to initiate activity in the run-time
    *  group.  For example, a "data message from above" can be sent
    *  to one of the run-times, causing the receiver to divide it up,
    *  distribute the pieces, then re-assemble them and return the
    *  result to the console.
    * A console has the same client and server as the other run-times,
    *  but the scheduler is replaced with something to generate 
    *  messages.
    * 
    * Build the client, the server, the scheduler, and the queues 
    * linking them. Then start the client, server, and scheduler going
    */
   public RTParentPeerImpl( short _portToListenOn, int _numRTs ) 
    {    // add self to the number of run-times when call super's
         //  constructor..  note, the console has no above-peer
      super( _portToListenOn, (short) 0, _numRTs + 1 );
      
      toClientQ    = new LinkedBlockingQueue( );
      toSchedQ     = new LinkedBlockingQueue( );

      theClient = new RTClientForObjectsThdd( toClientQ );
      theClient.start();

      theServer = new RTServerForObjectsThdd( toSchedQ, 
       portToListenOn );
      theServer.start();

      theScheduler = new RTSchedForParent( 
       toClientQ, null, toSchedQ, (RTPeer)this );
      theScheduler.start();
    }
 }
