/*
* RTSchedForObjectsThdd.java
*
* Created on May 24, 2005, 1:07 AM
*
*/

package scheduler;

import java.io.*;
import java.util.concurrent.*;
import java.awt.*;
import message.*;
import server.RTServer;
import worker.RTWorker;
import client.RTClient;
import main.*;
import peer.RTPeer;

/**
*
* @author Kevinsean Halle
*  May 29, 6pm..  first version of console,
 *  Receives an "add this runtime to your group" message for each
 *   runtime in the group.  The message has the name and port of the
 *   runtime to add. 
 *  Then receives a "group complete" message
 *  That triggers it to create a "data message from above" message and
 *   send it to one of the run-times in the group (chosen at random)
 *  Needs to know the ports (addresses) of the other run-times in group 
*/
public class RTSchedForParent extends RTScheduler
 { protected BlockingQueue toClientQ;
   protected BlockingQueue toSelfQ;
   protected RTPeer        parentRT;
   protected int           howManyRunTimesInGroup;
   protected RTMsgWindow   msgWindow;
   
      // helper vars for helper methods at end
   protected boolean       keepTrying;
   protected RTMsgBase         msg;
   
   /** Creates a new instance of RTSchedForObjectsThdd */
   public RTSchedForParent( 
    BlockingQueue _toClientQ, BlockingQueue _toWorkerQ, 
    BlockingQueue _toSelfQ, RTPeer _parentRT ) 
    {
      super( _toClientQ, _toWorkerQ, _toSelfQ, _parentRT );
      
      toClientQ   = _toClientQ;
      toSelfQ     = _toSelfQ;
      parentRT    = _parentRT;

      msgWindow = new RTMsgWindow("Sched of Parent at Port: " + 
       parentRT.getPort() );

    }
 
   public void run()
    { RTMsgSub msgToSend;  //compiler complains about init
      RTMsgBase msgIn;
      
      // create a message, put in the dest run-time's port.  
      // give it to the client..  
      // it should get divided up by the recipient, a piece done by
      //  each member of the group, the results put back together and
      //  returned to here.

         // create the message
      RTMsgSub starterMsg = makeStarterReqChainDataTakeMsg();
      
      msgToSend = makeReqChainDataTakeMsgFromAbove( starterMsg );
      
      msgToSend.setUndivider( (short)(9900 + randNumGen.nextInt(3)));
      msgToSend.setDataSize( 633 );
      
      
      msgWindow.displayMsg( "Sending to all: ", msgToSend );
      
         // Send by placing on the queue to the client
      sendMsgToAllInGroup( msgToSend );
      
         // Go into normal message handling loop
      while( true )
       {    
            //get next thing in the queue (from server or worker)        
         msgIn = getNextMsgIn();   
         
         msgWindow.displayMsg( "Got ", msgIn );

            // figure out what kind of message it is and act accdingly
         dispatchMsg( msgIn );
                  
       } 

    }

   /**  wherever the data-piece happens to be, this message
        shows up..  for now, it goes to the console..  because the
        console is acting as everything that's not in the peer
        group.
       Simply create a response message..  the real message 
        would have the data payload, but this don't.  So the 
        response is a clone, but with different types..
    */
   protected void handleReqChainDataGiveMsgFromAny( RTMsgBase msgIn )
    {
      RTMsgSub msgOut = makeRespDataTakeMsgFromAny( msgIn );
      sendMsg( msgOut );
    }
   
   
   /** Get this anytime one of the peers in the group below finishes
    *   computing something.  It lets the above-peer track the status
    *   of all the peers.  It uses this status to decide which to name
    *   as the un-divider.
    */
   protected void handleInfoComplMsgFromBelow( RTMsgBase msgIn )
    {
      // normally would update the status table, but this version
      //  of the code has no actual scheduling alg, so nothing
      //  to do..
    }
   
   /** 
    *  Don't need to do anything..  the console receiving this means
    *   the experiment was a success!
    */
   protected void handleRespDataTakeMsgFromBelow( RTMsgBase msgIn )
    {
      msgWindow.displayText("\n\nHandling response from undivider");
      msgWindow.displayText(" Which means the test succeeded.\n");    }
   
   
   /*
    *  Override the standard error handler, so that it uses the
    *   message window.  The message window is created here in 
    *   the sub-class.  
    *   Any time the combination of types
    *   is invalid, end up here..  print a message to the msgWindow
    */
   protected void handleMsgError( RTMsgBase msgIn )
    {
      msgWindow.displayMsg ("An erroneous msg came in", msgIn );
    }
      
 }
