
package server;

import java.io.*;
import java.net.*;
import java.util.concurrent.*;
import message.RTMsgBase;
import main.*;

/**
 * make a ServerSocket and wait for connections.
 * @author Sean Halle
 */

public class RTServerForObjectsThdd extends RTServer
 { 
   
      /** The TCP port for the service. */
   private short thisPort;
   private BlockingQueue servToSchedQ;
   
   public RTServerForObjectsThdd( BlockingQueue _servToSchedQ, 
    short _thisPort )
    { servToSchedQ = _servToSchedQ;
      thisPort = _thisPort;
    }
   
   public void run()
    { ServerSocket socketFactory;
      Socket  msgSocket;
      try 
       { socketFactory = new ServerSocket( thisPort );
         
            //each new message coming in generates its own socket
            // the message is taken off that socket, then go back
            // and wait for another message to come in..
         while ((msgSocket = socketFactory.accept()) != null) 
          {    // Get the message and do what needs to be done with it
            handleMessage(msgSocket);
          }

       } 
      catch (IOException e) 
       { System.err.println(e);
       }
    }

   /** Model is that each run-time has one server, one client.
        The client only sends messages to other servers.
        The server only accepts messages from clients.
        Each message a client sends generates a new socket.
        This method pulls the message off the socket.
        Then it hands the message off to the scheduler.
        Hands off via a non-blocking queue.*/
   private void handleMessage(Socket sock) throws IOException 
    {
      try
       {  ObjectInputStream is = new ObjectInputStream(new 
          BufferedInputStream(sock.getInputStream()));

            // Read and validate the Object
         Object o = is.readObject();
         if (o == null) 
          { System.err.println("Read null from server!");
            sock.close();
          } 
         else if ((o instanceof RTMsgBase)) 
          {    // Valid, so cast to RTMsgBase, and put into queue
            RTMsgBase msgRecvd = (RTMsgBase) o;
               // send the message to the scheduler
            boolean keepTrying = true;
            while (keepTrying == true)
             {
               try
                { servToSchedQ.put(msgRecvd);
                  keepTrying = false;
                }
               catch (InterruptedException e)
                {    // means got interrupted while waiting to put into queue.. 
                     // leave keepTrying true..
                  keepTrying = true;
                }
             }
            sock.close();
          } 
         else 
          { throw new IllegalArgumentException("Wanted RTMsg, got " + o);
          }
       } 
      catch (ClassNotFoundException e) 
       { System.err.println("Wanted RTMsg, got INVALID CLASS (" + e + ")");
       } 
      catch (IOException e) 
       { System.err.println(e);
       }
    }
 }


